VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CRMResponse"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Attribute VB_Ext_KEY = "SavedWithClassBuilder6" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"
'Concourse Suite Outlook-Plugin
'Copyright (C) 2000-2007 Concursive Corporation.
'
'This program is free software; you can redistribute it and/or
'modify it under the terms of the GNU General Public License
'as published by the Free Software Foundation; version 2
'of the License.
'
'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'GNU General Public License for more details.
'
'You should have received a copy of the GNU General Public License
'along with this program; if not, write to the Free Software
'Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

Option Explicit
Option Base 0

' CRMResponse class
'
' Represents Concourse Suite XML API response.

' Response XML tag constants
Private Const TAG_RESPONSE_ROOT = "aspcfs"
Private Const TAG_RESPONSE = "response"
Private Const TAG_ERROR_CODE = "status"
Private Const TAG_ERROR_TEXT = "errorText"
Private Const TAG_RECORD_SET = "recordSet"
Private Const TAG_RECORD = "record"

' Response XML attribute constants
Private Const ATT_RESPONSE_ID = "id"
Private Const ATT_ACTION = "action"
Private Const ATT_NAME = "name"

Private m_xml As String
Private m_results As New Collection

' Response XML text. Default property.
Public Property Get XML() As Variant
Attribute XML.VB_UserMemId = 0
    XML = m_xml
End Property

' Response XML text setter. Parses XML text and build results collection.
' As long as this property made default, the next two fragments of code are equivalent:
'
' 1.
' response.XML = textXML
'
' 2.
' response = textXML
'
Public Property Let XML(value As Variant)
    Debug.Print value
    m_xml = value
    
    ParseXml (value)
End Property

' Collection of the response results.
'
' Results could be accessed by index (in order they appear in response XML)
' or enumerated using FOR EACH statement.
'
Public Property Get Results() As Collection
    Set Results = m_results
End Property

' Get result by its transaction ID
'
Public Property Get ResultSetByID(TransactionID As Integer) As CRMResultSet
    On Error Resume Next
    Set ResultSetByID = Results(str$(TransactionID))
End Property

Private Sub ParseXml(value As String)
    Set m_results = New Collection

    Dim document As New MSXML.DOMDocument
    If document Is Nothing Then
        CentricApp.Logger.Debugging "Couldn't create object: MSXML.DOMDocument."
        Err.Raise ERR_UNEXPECTED, Description:=MSG_ERR_UNEXPECTED & vbCrLf & _
        "Couldn't create object: MSXML.DOMDocument."
    End If
    
    If Not document.LoadXML(value) Then
        Err.Raise ERR_XML_PARSING_ERROR, Description:=document.parseError.reason
    End If
    
    Dim root As IXMLDOMNode
    Set root = document.SelectSingleNode(TAG_RESPONSE_ROOT)
    If IsEmpty(root) Then
        Err.Raise ERR_MISSING_TAG
    End If
    
    Dim nodeList As IXMLDOMNodeList
    Set nodeList = root.SelectNodes(TAG_RESPONSE)
    
    Dim node As IXMLDOMNode
    For Each node In nodeList
        If TypeName(node) = "IXMLDOMElement" Then
            Dim element As IXMLDOMElement
            Set element = node
            
            Dim numID As Integer
            If IsNull(element.getAttribute(ATT_RESPONSE_ID)) Then
                numID = 0
            Else
                Dim strID As String
                strID = element.getAttribute(ATT_RESPONSE_ID)
                numID = Val(strID)
            End If
            
            Dim child As IXMLDOMNode
            Set child = node.SelectSingleNode(TAG_ERROR_CODE)
            Dim strErrCode As String
            Dim errCode As Integer
            If Not IsNull(child) Then
                strErrCode = child.text
                errCode = Val(strErrCode)
            End If
            
            Set child = node.SelectSingleNode(TAG_ERROR_TEXT)
            Dim errText As String
            If Not IsNull(child) Then
                errText = child.text
            End If
            
            Dim resultSet As CRMResultSet
            Set resultSet = New CRMResultSet
            If errCode = 0 Then
                Set child = node.SelectSingleNode(TAG_RECORD_SET)
                If child Is Nothing Then
                    resultSet.Init numID, errCode, errText, "", New Collection
                Else
                    Set element = child
                    Dim src As String
                    src = element.getAttribute(ATT_NAME)
                    
                    Dim rows As Collection
                    Set rows = GetResultRows(child)
                
                    resultSet.Init numID, errCode, errText, src, rows
                End If
            Else
                resultSet.Init numID, errCode, errText, "", Nothing
            End If
            If numID = 0 Then
                m_results.Add resultSet
            Else
                m_results.Add resultSet, str$(numID)
            End If
        End If
    Next node
End Sub


Private Function GetResultRows(recordsetNode As IXMLDOMNode) As Collection
    Dim result As New Collection
    Dim list As IXMLDOMNodeList
    Set list = recordsetNode.SelectNodes(TAG_RECORD)
    If (IsNull(recordsetNode) Or (TypeName(recordsetNode) <> "IXMLDOMElement") Or (list.Length = 0)) Then
    Else
        Dim child As IXMLDOMNode
        For Each child In list
            Dim element As IXMLDOMElement
            Set element = child
            Dim Action As Integer
            ' Missing action attribute means SELECT
            If IsNull(element.getAttribute(ATT_ACTION)) Then
                Action = opSelect
            Else
                Dim strAction As String
                strAction = element.getAttribute(ATT_ACTION)
                Action = GetActionFromString(strAction)
            End If
            
            Dim Items As Collection
            Set Items = GetResultItems(child)
            
            Dim row As CRMResultRow
            Set row = New CRMResultRow
            row.Init Action, Items
            result.Add row
        Next child
    End If
        
    Set GetResultRows = result
End Function


Private Function GetResultItems(recordsetNode As IXMLDOMNode) As Collection
    Dim result As New Collection
    If (IsNull(recordsetNode) Or (TypeName(recordsetNode) <> "IXMLDOMElement") _
        Or (recordsetNode.ChildNodes.Length = 0)) Then
    Else
        Dim element As IXMLDOMElement
        Set element = recordsetNode
        
        Dim child As IXMLDOMNode
        For Each child In element.ChildNodes
            Dim Item As CRMResultItem
            Set Item = New CRMResultItem
            Item.Init child.nodeName, child.text
            result.Add Item, Item.Name
        Next child
    End If
        
    Set GetResultItems = result
End Function

Private Function GetActionFromString(ByVal str As String) As Integer
    Dim result As Integer
    result = opUnknown
    
    Select Case str
        Case "select"
            result = opSelect
        Case "insert"
            result = opInsert
        Case "update"
            result = opUpdate
        Case "delete"
            result = opDelete
    End Select
    
    GetActionFromString = result
End Function
