Attribute VB_Name = "Communication"
'Concourse Suite Outlook-Plugin
'Copyright (C) 2000-2007 Concursive Corporation.
'
'This program is free software; you can redistribute it and/or
'modify it under the terms of the GNU General Public License
'as published by the Free Software Foundation; version 2
'of the License.
'
'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'GNU General Public License for more details.
'
'You should have received a copy of the GNU General Public License
'along with this program; if not, write to the Free Software
'Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 

Option Explicit
Option Base 0

Private m_NextTransactionID As Integer

Public Const CONNECTION_PARAM_URL = "url"
Public Const CONNECTION_PARAM_USER_NAME = "user_name"
Public Const CONNECTION_PARAM_PASSWORD = "password"

' CRM packet processing page address
Const PAGE_PACKET_PROCESSING = "ProcessPacket.do"


Public Function CreateConnectionParamsWithBasicAuthentication(Url As String, UserName As String, Password As String)
    Dim params As New ConnectionParams
    
    params(CONNECTION_PARAM_URL) = Url
    params(CONNECTION_PARAM_USER_NAME) = UserName
    params(CONNECTION_PARAM_PASSWORD) = Password
    
    Set CreateConnectionParamsWithBasicAuthentication = params
End Function

Public Function NextTransactionID() As Integer
    If m_NextTransactionID = 32767 Then
        m_NextTransactionID = 0
    End If
    m_NextTransactionID = m_NextTransactionID + 1
    NextTransactionID = m_NextTransactionID
End Function

Public Function GetUserId(ByVal params As ConnectionParams, ByVal UserName As String) As String
    CentricApp.Logger.Debugging "Getting user ID (" & UserName & ")."
    
    Dim op As New CRMOperation
    op.Action = opSelect
    op.Target = "userList"
    op.Conditions.Add "username", UserName
    
    Dim transaction As New CRMTransaction
    transaction.Properties.Add "id"
    transaction.Add op
    Dim req As New CRMRequest
    req.Add transaction
    
    Dim conn As New CRMConnection
    Set conn.ConnectionParams = params
    
    Dim response As New CRMResponse
    response = conn.Send(req, 60)
    
    Dim rs As CRMResultSet
    Set rs = response.Results(1)
    If rs.IsError Then
        Err.Raise rs.ErrorCode, Description:=rs.ErrorText
    End If

    GetUserId = rs.Items(1)("id")
    CentricApp.Logger.Debugging "User ID found: " & GetUserId & "."
End Function


Public Function TestConnection(ByVal params As ConnectionParams) As Boolean
    CentricApp.Logger.Debugging "Testing connection (executing test request) - " & _
    "USER NAME: " & params(CONNECTION_PARAM_USER_NAME) & " " & _
    "PASSWORD: " & params(CONNECTION_PARAM_PASSWORD) & " " & _
    "SERVER URL: " & params(CONNECTION_PARAM_URL)
    
    On Error GoTo handleError
    
    GetUserId params, params(CONNECTION_PARAM_USER_NAME)
    
    TestConnection = True
    CentricApp.Logger.Debugging "Connection test passed."
    Exit Function
    
handleError:
    TestConnection = False
    CentricApp.Logger.Debugging "Connection test failed."
End Function

Public Function GetTimestamp(ByVal moduleName As String, ByVal objectID As String) As String
    CentricApp.Logger.Debugging "Getting timestamp value for " & moduleName & " id=" & objectID

    Dim op As New CRMOperation
    op.Action = opSelect
    op.Target = moduleName
    op.Conditions.Add "id", objectID
    
    Dim transaction As New CRMTransaction
    transaction.Properties.Add "modified"
    transaction.Add op
    
    Dim req As New CRMRequest
    req.Add transaction
    
    Dim conn As New CRMConnection
    Set conn.ConnectionParams = CreateConnectionParamsWithBasicAuthentication( _
        CentricApp.Preferences.Url, _
        CentricApp.Preferences.UserName, _
        CentricApp.Preferences.Password _
    )
    
    Dim response As New CRMResponse
    response = conn.Send(req, 60)
    Dim rs As CRMResultSet
    Set rs = response.ResultSetByID(transaction.ID)
    
    If rs.Items.Count <= 0 Then
        CentricApp.Logger.Debugging "Error getting timestamp: " & _
        "Item #" & objectID & " from " & moduleName & " not found in CCRM."
        
        Err.Raise ERR_ITEM_NOT_FOUND, Description:="Item #" & objectID & " from " & moduleName & " not found in CCRM."
    End If
    
    GetTimestamp = rs.Items(1)("modified")
End Function



