Attribute VB_Name = "ContactUtils"
'Concourse Suite Outlook-Plugin
'Copyright (C) 2000-2007 Concursive Corporation.
'
'This program is free software; you can redistribute it and/or
'modify it under the terms of the GNU General Public License
'as published by the Free Software Foundation; version 2
'of the License.
'
'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'GNU General Public License for more details.
'
'You should have received a copy of the GNU General Public License
'along with this program; if not, write to the Free Software
'Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 

Option Explicit

Public Enum PhoneType
    ptBusiness = 1
    ptBusiness2 = 2
    ptBusinessFax = 3
    ptHome = 4
    ptHome2 = 5
    ptHomeFax = 6
    ptMobile = 7
    ptPager = 8
    ptOther = 9
End Enum

Public ContactManager As New ContactManager

Private m_CurrentContact As New ContactInfo

Public Property Get CurrentContact() As ContactInfo
    Set CurrentContact = m_CurrentContact
End Property

Public Sub SetCurrentContact(ByVal Name As String, ByVal Email As String)
    With CurrentContact
        .Company = ""
        .EmailAddress = Email
        .FirstName = ""
        .ID = -1
        .LastName = ""
        .PhoneNumber = ""
    End With
    ParseContactName (Name)
End Sub

Public Function IsValidContactItem(ByVal contact As ContactItem, ByRef ErrorTexts As String) As Boolean
    CentricApp.Logger.Debugging "Validating contact (" & contact.FileAs & ")"
    
    IsValidContactItem = True
    ErrorTexts = ""
    
    If contact.Email1Address = "" Then
        ErrorTexts = ErrorTexts & MSG_REQ_ADD_CONTACT_EMAIL & vbCrLf
        IsValidContactItem = False
        CentricApp.Logger.Debugging "Contact email is missing."
    End If
    
    'If contact.Email1AddressType <> "SMTP" Then
        'ErrorTexts = ErrorTexts & MSG_REQ_ADD_CONTACT_EMAIL_TYPE & vbCrLf
        'IsValidContactItem = False
        'CentricApp.Logger.Debugging "Invalid contact mail address type: " & contact.Email1AddressType
    'End If
    
    If contact.LastName = "" And contact.CompanyName = "" Then
        ErrorTexts = ErrorTexts & MSG_REQ_ADD_CONTACT_NAME_OR_COMPANY & vbCrLf
        IsValidContactItem = False
        CentricApp.Logger.Debugging "Missing contact last name or company. (Last name: " & _
        contact.LastName & ", company: " & contact.CompanyName & ")"
    End If
    
    If IsValidContactItem Then
        CentricApp.Logger.Debugging "Contact is OK (" & contact.FileAs & ")"
    End If
End Function

Public Function GetContactList(searchText As String) As Collection
    CentricApp.Logger.Debugging "Getting contact list (search text is: " & searchText & ")"
    
    Dim request As New CRMRequest
    request.Add CreateSearchTransaction("emailAddress", searchText)
    request.Add CreateSearchTransaction("lastName", searchText)
    request.Add CreateSearchTransaction("firstName", searchText)

    Dim conn As New CRMConnection
    Set conn.ConnectionParams = CreateConnectionParamsWithBasicAuthentication( _
        CentricApp.Preferences.Url, _
        CentricApp.Preferences.UserName, _
        CentricApp.Preferences.Password _
    )
    
    Dim response As New CRMResponse
    response = conn.Send(request, 60)

    Dim dictionary As New dictionary

    Dim rs As CRMResultSet
    For Each rs In response.Results
        If rs.IsError Then
            Err.Raise rs.ErrorCode, Description:=rs.ErrorText
        End If
        Dim row As CRMResultRow
        For Each row In rs.Items
            Dim ID As Integer
            ID = Val(row("id"))
            If Not dictionary.Exists(ID) Then
                dictionary.Add ID, CreateContactInfo(row)
            End If
        Next row
    Next rs

    CentricApp.Logger.Debugging str$(UBound(dictionary.Items) - LBound(dictionary.Items)) & _
    " contacts found."
    
    Dim result As New Collection
    Dim contact As ContactInfo
    Dim i As Integer
    For i = LBound(dictionary.Items) To UBound(dictionary.Items)
        Set contact = dictionary.Items(i)
        result.Add contact, contact.ID
    Next i
    
    Set GetContactList = result
End Function

Public Function CreateCRMContact(ByVal FirstName As String, ByVal MiddleName As String, _
    ByVal LastName As String, ByVal Company As String, ByVal Title As String, _
    ByVal Role As String, ByVal Email As String) As ContactInfo

    CentricApp.Logger.Debugging "Creating new CRM contact (first name:" & FirstName & _
    ", middlename: " & MiddleName & ", last name: " & LastName & ", company: " & Company & _
    ", title: " & Title & ", role: " & Role & ", email: " & Email & ")"
    
    Dim ID As Integer
    ID = CentricApp.UserID
    
    Dim transaction As New CRMTransaction
    transaction.Properties.Add "id"
    
    Dim op As New CRMOperation
    op.Action = opInsert
    op.Target = "contact"
    op.ShareKey = True
    With op.Conditions
        .Add "disableNameValidation", True
        .Add "company", Company
        .Add "orgName", Company
        .Add "title", Title
        .Add "nameFirst", FirstName
        .Add "nameMiddle", MiddleName
        .Add "nameLast", LastName
        .Add "role", Role
        .Add "accessType", 2
        .Add "enteredBy", ID
        .Add "modifiedBy", ID
        .Add "owner", ID
        If CentricApp.Preferences.ImportContactAsLead Then
            .Add "isLead", "true"
            .Add "leadStatus", 2
        End If
End With
    transaction.Add op
    
    Set op = GetContactEmailOperation(Email)
    transaction.Add op
    
    Dim request As New CRMRequest
    request.Add transaction
    
    Dim conn As New CRMConnection
    Set conn.ConnectionParams = CreateConnectionParamsWithBasicAuthentication( _
        CentricApp.Preferences.Url, _
        CentricApp.Preferences.UserName, _
        CentricApp.Preferences.Password _
    )
    
    Dim response As New CRMResponse
    response = conn.Send(request, 60)
    
    Dim rs As CRMResultSet
    Set rs = response.ResultSetByID(transaction.ID)
    
    If rs.IsError Then
        Err.Raise rs.ErrorCode, Description:=rs.ErrorText
    End If
    
    Dim contact As New ContactInfo
    contact.ID = Val(rs(1)("id"))
    contact.Company = Company
    contact.EmailAddress = Email
    contact.FirstName = FirstName
    contact.LastName = LastName
    
    CentricApp.Logger.Debugging "CRM contact created with ID=" & contact.ID
    
    Set CreateCRMContact = contact
End Function

' Request contact ID by email address from CRM
Public Function FindContactIDFromEmail(ByVal Email As String) As Integer
    CentricApp.Logger.Debugging "Searching existing contact ID by email (" & Email & ")"
    
    Dim transaction As New CRMTransaction
    With transaction.Properties
        .Add "contactId"
    End With
    
    Dim op As New CRMOperation
    op.Action = opSelect
    op.Target = "contactEmailAddressList"
    op.Conditions.Add "emailAddress", Email

    transaction.Add op
    
    Dim request As New CRMRequest
    request.Add transaction

    Dim conn As New CRMConnection
    Set conn.ConnectionParams = CreateConnectionParamsWithBasicAuthentication( _
        CentricApp.Preferences.Url, _
        CentricApp.Preferences.UserName, _
        CentricApp.Preferences.Password _
    )
    
    Dim response As New CRMResponse
    response = conn.Send(request, 60)

    Dim rs As CRMResultSet
    Set rs = response.ResultSetByID(transaction.ID)
    If rs.IsError Then
        Err.Raise rs.ErrorCode, Description:=rs.ErrorText
    End If

    If (rs.Items.Count <= 0) Then
        CentricApp.Logger.Debugging "Contact not found."
        Err.Raise ERR_RESULTSET_IS_EMPTY
    End If
    FindContactIDFromEmail = rs.Items(1)("contactId")
    CentricApp.Logger.Debugging "Contact found, ID=" & FindContactIDFromEmail
End Function

' Get existing CRM contact by email address
' or search the contact using serach form
' or create new one
Public Function GetContactIDOrCreateWithUI(ByVal Email As String) As Integer
    On Error GoTo handleError
    
    GetContactIDOrCreateWithUI = FindContactIDFromEmail(Email)
    Exit Function
    
handleError:
    If Err.Number = ERR_RESULTSET_IS_EMPTY Then
        frmSearchContacts.Show vbModal
        If frmSearchContacts.ButtonPressed = IgnoreContact Then
            Unload frmSearchContacts
            Err.Raise ERR_IGNORE_CONTACT, Description:=MSG_ERR_IGNORE_CONTACT
        End If
        If frmSearchContacts.ButtonPressed = CancelSync Then
            Unload frmSearchContacts
            Err.Raise ERR_ABORTED_BY_USER, Description:=MSG_ERR_ABORTED_BY_USER
        End If
        If frmSearchContacts.ButtonPressed = SelectContact Then
            GetContactIDOrCreateWithUI = frmSearchContacts.ContactID
            
            Dim ContactID As Integer
            'Dim Email As String
            ContactID = frmSearchContacts.ContactID
            Email = frmSearchContacts.Email
            
            Unload frmSearchContacts
            
            If LCase$(Email) <> LCase$(CurrentContact.EmailAddress) Then
                Call CreateSecondaryEmail(ContactID, CurrentContact.EmailAddress)
            End If
            
            If IsBatchMode Then
                ContactManager.CacheEmail Email, CStr(ContactID)
            End If
            Exit Function
        End If
    End If
End Function

Public Function GetContactID(ByVal Email As String)
    If IsBatchMode Then
        If ContactManager.IsContactExists(Email) Then
            GetContactID = ContactManager.GetContactID(Email)
        Else
            GetContactID = GetContactIDOrCreateWithUI(Email)
        End If
    Else
        GetContactID = GetContactIDOrCreateWithUI(Email)
    End If
End Function

Private Function CreateContactInfo(ByVal row As CRMResultRow) As ContactInfo
    Dim result As New ContactInfo
    result.ID = Val(row("id"))
    result.FirstName = row("nameFirst")
    result.LastName = row("nameLast")
    result.Company = row("company")
    result.EmailAddress = row("primaryEmailAddress")
    result.PhoneNumber = row("primaryPhoneNumber")
    Set CreateContactInfo = result
End Function

Private Function CreateSearchTransaction(ByVal field As String, ByVal searchText As String) As CRMTransaction
    Dim transaction As New CRMTransaction
    With transaction.Properties
        .Add "id"
        .Add "nameFirst"
        .Add "nameLast"
        .Add "company"
        .Add "primaryEmailAddress"
        .Add "primaryPhoneNumber"
    End With
    
    searchText = searchText & "%"
    
    Dim op As New CRMOperation
    op.Action = opSelect
    op.Target = "contactList"
    op.Conditions.Add field, searchText
    op.Conditions.Add "employeesOnly", 0
    op.Conditions.Add "includeEnabled", 1
    transaction.Add op
    Set CreateSearchTransaction = transaction
End Function

Public Function GetContactAddressOperation(street As String, city As String, state As String, _
    zip As String, country As String, Optional ContactID As Variant) As CRMOperation
    
    Dim ID As Integer
    ID = CentricApp.UserID
    
    Dim op As New CRMOperation
    op.Action = opInsert
    op.Target = "contactAddress"
    With op.Conditions
        If IsMissing(ContactID) Then
            .Add "contactId", "$C{contact.id}"
        Else
            .Add "contactId", ContactID
        End If
        .Add "streetAddressLine1", street
        .Add "streetAddressLine2", ""
        .Add "city", city
        .Add "state", state
        .Add "zip", zip
        .Add "country", country
        .Add "enteredBy", ID
        .Add "modifiedBy", ID
        .Add "enabled", "true"
        .Add "entered", 0
        .Add "type", 1
    End With
    Set GetContactAddressOperation = op
End Function

Public Function GetContactEmailOperation(Email As String, Optional ContactID As Variant) As CRMOperation
    Dim ID As Integer
    ID = CentricApp.UserID
    
    Dim op As New CRMOperation
    op.Action = opInsert
    op.Target = "contactEmailAddress"
    With op.Conditions
        If IsMissing(ContactID) Then
            .Add "contactId", "$C{contact.id}"
        Else
            .Add "contactId", ContactID
        End If
        .Add "email", Email
        .Add "enteredBy", ID
        .Add "modifiedBy", ID
        .Add "enabled", "true"
        .Add "entered", ID
        .Add "modified", ID
        .Add "type", 1
    End With
    Set GetContactEmailOperation = op
End Function

Public Function GetContactPhoneOperation(phone As String, PhoneType As PhoneType, Optional ContactID As Variant) As CRMOperation
    Dim ID As Integer
    ID = CentricApp.UserID
    
    Dim op As New CRMOperation
    op.Action = opInsert
    op.Target = "contactPhoneNumber"
    With op.Conditions
        If IsMissing(ContactID) Then
            .Add "contactId", "$C{contact.id}"
        Else
            .Add "contactId", ContactID
        End If
        .Add "number", phone
        .Add "extension", ""
        .Add "enteredBy", ID
        .Add "modifiedBy", ID
        .Add "enabled", "true"
        .Add "entered", 0
        .Add "modified", 0
        .Add "type", PhoneType
    End With
    Set GetContactPhoneOperation = op
End Function

Private Sub ParseContactName(ByVal Name As String)
    Dim pos As Integer
    Dim part As String
    
    pos = InStr(1, Name, ",", vbTextCompare)
    If pos >= 1 Then
        part = Left$(Name, pos - 1)
        CurrentContact.LastName = Trim$(part)
        part = Right$(Name, Len(Name) - pos - 1)
        CurrentContact.FirstName = Trim$(part)
        Exit Sub
    End If
    
    pos = InStr(1, Name, " ", vbTextCompare)
    If pos >= 1 Then
        part = Left$(Name, pos - 1)
        CurrentContact.FirstName = Trim$(part)
        part = Right$(Name, Len(Name) - pos)
        CurrentContact.LastName = Trim$(part)
        Exit Sub
    End If
    
    CurrentContact.FirstName = Trim$(Name)
    CurrentContact.LastName = ""
End Sub

Public Function GetContactIDByUserID(ByVal UserID As Integer) As Integer
    CentricApp.Logger.Debugging "Finding contactID for user ID=" & UserID & "..."
    
    Dim request As New CRMRequest
    Dim transaction As New CRMTransaction
    transaction.Properties.Add ("contactId")
    transaction.Properties.Add ("id")
    
    Dim op As New CRMOperation
    op.Action = opSelect
    op.Target = "userList"
    op.Conditions.Add "id", UserID
    transaction.Operations.Add op
    
    request.Add transaction
    
    Dim conn As New CRMConnection
    Set conn.ConnectionParams = CreateConnectionParamsWithBasicAuthentication( _
        CentricApp.Preferences.Url, _
        CentricApp.Preferences.UserName, _
        CentricApp.Preferences.Password _
    )
    
    Dim response As New CRMResponse
    response = conn.Send(request, 60)
    
    Dim rs As CRMResultSet
    Set rs = response.ResultSetByID(transaction.ID)
    
    If rs.IsError Then
        CentricApp.Logger.Debugging "Error getting contactID."
        Err.Raise rs.ErrorCode, Description:=rs.ErrorText
    End If
    
    GetContactIDByUserID = Val(rs(1)("contactId"))
    CentricApp.Logger.Debugging "Contact found, ID=" & GetContactIDByUserID
End Function

Private Sub CreateSecondaryEmail(ByVal ContactID As String, ByVal Email As String)
    CentricApp.Logger.Debugging "Creating secondary email address for contact ID=" & ContactID & " (" & Email & ")..."
    
    Dim request As New CRMRequest
    Dim transaction As New CRMTransaction
    transaction.Add GetContactEmailOperation(Email, ContactID)
    
    request.Add transaction
    
    Dim conn As New CRMConnection
    Set conn.ConnectionParams = CreateConnectionParamsWithBasicAuthentication( _
        CentricApp.Preferences.Url, _
        CentricApp.Preferences.UserName, _
        CentricApp.Preferences.Password _
    )
    
    Dim response As New CRMResponse
    response = conn.Send(request, 60)
    
    Dim rs As CRMResultSet
    Set rs = response.ResultSetByID(transaction.ID)
    
    If rs.IsError Then
        CentricApp.Logger.Debugging "Error creating secondary email."
        Err.Raise rs.ErrorCode, Description:=rs.ErrorText
    End If
    CentricApp.Logger.Debugging "Secondary email sucessfully created."
End Sub
