Attribute VB_Name = "Main"
'Concourse Suite Outlook-Plugin
'Copyright (C) 2000-2007 Concursive Corporation.
'
'This program is free software; you can redistribute it and/or
'modify it under the terms of the GNU General Public License
'as published by the Free Software Foundation; version 2
'of the License.
'
'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'GNU General Public License for more details.
'
'You should have received a copy of the GNU General Public License
'along with this program; if not, write to the Free Software
'Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 

Option Explicit


Public CentricApp As New ConcourseSuite.CentricApp

Public Sub SendContact(ByVal contact As ContactItem)
    CentricApp.Logger.Debugging "Sending contact (" & contact.FileAs & ")."
    
    Dim SyncItem As SyncItem
    Set SyncItem = New SyncItem
    Set SyncItem.OutlookItem = contact
    
    If SyncItem.IsSynchronized Then
        CentricApp.Logger.Debugging "Contact already synchronized (" & contact.FileAs & ")."
        MsgBox MSG_ERR_ALREADY_SYNCHRONIZED, vbExclamation, TITLE_ADD_CONTACT
        Exit Sub
    End If

    Dim SyncResult As SyncResult
    Set SyncResult = ProcessSyncItem(SyncItem)
    LogSyncResult SyncResult
    DisplaySyncResult SyncResult, TITLE_ADD_CONTACT
    
End Sub

Public Sub SendTask(ByVal Task As TaskItem)
    CentricApp.Logger.Debugging "Sending task (" & Task.Subject & ")."
    
    Dim SyncItem As SyncItem
    Set SyncItem = New SyncItem
    Set SyncItem.OutlookItem = Task
    
    If SyncItem.IsSynchronized Then
        CentricApp.Logger.Debugging "Task already synchronized (" & Task.Subject & ")."
        MsgBox MSG_ERR_ALREADY_SYNCHRONIZED, vbExclamation, TITLE_ADD_TASK
        Exit Sub
    End If

    Dim SyncResult As SyncResult
    Set SyncResult = ProcessSyncItem(SyncItem)
    LogSyncResult SyncResult
    DisplaySyncResult SyncResult, TITLE_ADD_TASK
End Sub

Public Sub SendMail(ByVal Mail As MailItem)
    CentricApp.Logger.Debugging "Sending mail (" & Mail.Subject & ")."
    
    Dim SyncItem As SyncItem
    Set SyncItem = New SyncItem
    Set SyncItem.OutlookItem = Mail
    
    If SyncItem.IsSynchronized Then
        CentricApp.Logger.Debugging "Email already synchronized (" & Mail.Subject & ")."
        If IsSentMail(Mail) Then
            MsgBox MSG_ERR_ALREADY_SYNCHRONIZED, vbExclamation, TITLE_ADD_SENT_MAIL
        Else
            MsgBox MSG_ERR_ALREADY_SYNCHRONIZED, vbExclamation, TITLE_ADD_RECEIVED_MAIL
        End If
        Exit Sub
    End If
    
    Dim SyncResult As SyncResult
    Set SyncResult = ProcessSyncItem(SyncItem)
    LogSyncResult SyncResult
        If IsSentMail(Mail) Then
            DisplaySyncResult SyncResult, TITLE_ADD_SENT_MAIL
        Else
            DisplaySyncResult SyncResult, TITLE_ADD_RECEIVED_MAIL
        End If
End Sub

Public Sub SendAppointment(ByVal Appointment As AppointmentItem)
    CentricApp.Logger.Debugging "Sending appointment (" & Appointment.Subject & ")."
    
    Dim SyncItem As SyncItem
    Set SyncItem = New SyncItem
    
    Set SyncItem.OutlookItem = Appointment
    
    If SyncItem.IsSynchronized Then
        CentricApp.Logger.Debugging "Appointment already synchronized (" & Appointment.Subject & ")."
        MsgBox MSG_ERR_ALREADY_SYNCHRONIZED, vbExclamation, TITLE_ADD_APPOINTMENT
        Exit Sub
    End If

    Dim SyncResult As SyncResult
    Set SyncResult = ProcessSyncItem(SyncItem)
    LogSyncResult SyncResult
    DisplaySyncResult SyncResult, TITLE_ADD_APPOINTMENT
End Sub

Public Function ProcessSyncItem(ByVal SyncItem As SyncItem) As SyncResult
    Dim MailItem As MailItem
    Dim ContactItem As ContactItem
    Dim TaskItem As TaskItem
    Dim AppointmentItem As AppointmentItem

    Dim result As New SyncResult
    If SyncItem.IsNew Then
        If SyncItem.Class = olContact Then
            Set ContactItem = SyncItem.OutlookItem
            Set result = InternalSendContact(ContactItem)
        ElseIf SyncItem.Class = olTask Then
            Set TaskItem = SyncItem.OutlookItem
            Set result = InternalSendTask(TaskItem)
        ElseIf SyncItem.Class = olMail Then
            Set MailItem = SyncItem.OutlookItem
            Set result = InternalSendMail(MailItem)
        ElseIf SyncItem.Class = olAppointment Then
            Set AppointmentItem = SyncItem.OutlookItem
            Set result = InternalSendAppointment(AppointmentItem)
        End If
    Else
        If SyncItem.Class = olContact Then
            Set ContactItem = SyncItem.OutlookItem
            Set result = InternalUpdateContact(ContactItem)
        ElseIf SyncItem.Class = olTask Then
            Set TaskItem = SyncItem.OutlookItem
            Set result = InternalUpdateTask(TaskItem)
        ElseIf SyncItem.Class = olMail Then
            Set MailItem = SyncItem.OutlookItem
            Set result = InternalUpdateMail(MailItem)
        ElseIf SyncItem.Class = olAppointment Then
            Set AppointmentItem = SyncItem.OutlookItem
            Set result = InternalUpdateAppointment(AppointmentItem)
        End If
    End If
    Set ProcessSyncItem = result
End Function

Private Function IsValidTaskItem(ByVal Task As TaskItem, ByRef ErrorTexts As String)
    CentricApp.Logger.Debugging "Validating task..."
    IsValidTaskItem = True
    ErrorTexts = ""
    CentricApp.Logger.Debugging "Task is valid."
End Function

Private Function IsValidMailItem(ByVal Mail As MailItem, ByRef ErrorTexts As String)
    CentricApp.Logger.Debugging "Validating mail..."
    IsValidMailItem = True
    ErrorTexts = ""
    
    If IsSentMail(Mail) Then
        'If Mail.SenderEmailType <> "SMTP" Then
        If Mail.Recipients(1).AddressEntry.Type <> "SMTP" Then
            IsValidMailItem = False
            ErrorTexts = ErrorTexts & MSG_REQ_ADD_MAIL_SENDER_TYPE & vbCrLf
            CentricApp.Logger.Debugging "Sender email address type is invalid: " & Mail.SenderEmailType
        End If
    Else
        If Trim(Mail.Subject) = "" Then
            IsValidMailItem = False
            ErrorTexts = ErrorTexts & MSG_REQ_ADD_MAIL_SUBJECT & vbCrLf
            CentricApp.Logger.Debugging "Message subject is missing."
        End If
        'If Trim(Mail.SenderEmailAddress) = "" Then
            'IsValidMailItem = False
            'errorTexts = errorTexts & MSG_REQ_ADD_MAIL_SENDER & vbCrLf
        'End If
        If Mail.SenderEmailType <> "SMTP" Then
            IsValidMailItem = False
            ErrorTexts = ErrorTexts & MSG_REQ_ADD_MAIL_SENDER_TYPE & vbCrLf
            CentricApp.Logger.Debugging "Sender email address type is invalid: " & Mail.SenderEmailType
        End If
    End If
End Function

Private Function IsValidAppointmentItem(ByVal Appointment As AppointmentItem, ByRef ErrorTexts As String)
    CentricApp.Logger.Debugging "Validating appointment..."
    IsValidAppointmentItem = True
    ErrorTexts = ""
    CentricApp.Logger.Debugging "Appointment is valid."
End Function

Private Function SendMailWithKnownId(ByVal Mail As MailItem, ByVal ID As Integer) As CRMResultSet
    CentricApp.Logger.Debugging "Sending mail, user ID: " & ID

    Dim transaction As CRMTransaction
    If IsSentMail(Mail) Then
        Set transaction = GetSentMailTransaction(Mail, ID)
    Else
        Set transaction = GetReceivedMailTransaction(Mail, ID)
    End If

    Dim request As New CRMRequest
    request.Add transaction
    
    Dim conn As New CRMConnection
    Set conn.ConnectionParams = CreateConnectionParamsWithBasicAuthentication( _
        CentricApp.Preferences.Url, _
        CentricApp.Preferences.UserName, _
        CentricApp.Preferences.Password _
    )
    
    Dim response As New CRMResponse
    response = conn.Send(request, 60)
    
    Set SendMailWithKnownId = response.ResultSetByID(transaction.ID)
End Function


Private Function InternalSendContact(ByVal contact As Outlook.ContactItem) As SyncResult
    On Error GoTo handleError

    Dim ErrorTexts As String
    If Not IsValidContactItem(contact, ErrorTexts) Then
        Err.Raise ERR_INVALID_ITEM, Description:=ErrorTexts
    End If

    Dim SyncItem As SyncItem
    Set SyncItem = New SyncItem
    Set SyncItem.OutlookItem = contact
    
    Dim ID As Integer
    ID = CentricApp.UserID
    
    
    Dim op As New CRMOperation
    op.Action = opInsert
    op.Target = "contact"
    op.ShareKey = True
    With op.Conditions
        .Add "disableNameValidation", True
        .Add "company", contact.CompanyName
        .Add "orgName", contact.CompanyName
        .Add "title", contact.Title
        .Add "nameSalutation", contact.Suffix
        .Add "nameFirst", contact.FirstName
        .Add "nameMiddle", contact.MiddleName
        .Add "nameLast", contact.LastName
        .Add "nameSuffix", ""
        .Add "notes", contact.Body
        .Add "accessType", 2
        .Add "enteredBy", ID
        .Add "modifiedBy", ID
        .Add "owner", ID
        If CentricApp.Preferences.ImportContactAsLead Then
            .Add "isLead", "true"
            .Add "leadStatus", 2
        End If
    End With
    
    Dim transaction As New CRMTransaction
    transaction.Properties.Add "id"
    transaction.Add op
    
    AddContactOptionalFields transaction, contact, False
    
    Dim request As New CRMRequest
    request.Add transaction
    
    Dim conn As New CRMConnection
    Set conn.ConnectionParams = CreateConnectionParamsWithBasicAuthentication( _
        CentricApp.Preferences.Url, _
        CentricApp.Preferences.UserName, _
        CentricApp.Preferences.Password _
    )
    
    Dim response As New CRMResponse
    response = conn.Send(request, 60)
    
    Dim rs As CRMResultSet
    Set rs = response.ResultSetByID(transaction.ID)
    
    Set InternalSendContact = GetSyncResult(rs, MSG_ADD_CONTACT_SUCCESS, MSG_ERR_ADD_CONTACT)
    
    If (rs.IsOK) Then
        SyncItem.ID = rs.Items(1)("id")
        SyncItem.Save
    End If
    
    Exit Function
    
handleError:
    Set InternalSendContact = GetSyncResultFromError(Err, MSG_ERR_ADD_CONTACT)
End Function

Private Function InternalSendTask(ByVal Task As TaskItem) As SyncResult
    On Error GoTo handleError

    Dim ErrorTexts As String
    If Not IsValidTaskItem(Task, ErrorTexts) Then
        Err.Raise ERR_INVALID_ITEM, Description:=ErrorTexts
    End If

    Dim SyncItem As SyncItem
    Set SyncItem = New SyncItem
    Set SyncItem.OutlookItem = Task
    
    Dim ID As Integer
    ID = CentricApp.UserID
    
    Dim op As New CRMOperation
    op.Action = opInsert
    op.Target = "task"
    With op.Conditions
        .Add "description", Task.Subject
        .Add "notes", Task.Body
        ' TODO: Use Outlook task priority instead of hardcoded value
        .Add "priority", 5
        .Add "enteredBy", ID
        .Add "modifiedBy", ID
        .Add "owner", ID
        .Add "complete", Task.Complete
        .Add "enabled", 1
        .Add "dueDate", DateAsString(Task.DueDate)
        .Add "completeDate", DateAsString(Task.DateCompleted)
        .Add "entered", 0
        .Add "modified", 0
    End With

    Dim transaction As New CRMTransaction
    transaction.Properties.Add "id"
    transaction.Add op
    Dim request As New CRMRequest
    request.Add transaction
    
    Dim conn As New CRMConnection
    Set conn.ConnectionParams = CreateConnectionParamsWithBasicAuthentication( _
        CentricApp.Preferences.Url, _
        CentricApp.Preferences.UserName, _
        CentricApp.Preferences.Password _
    )
    
    Dim response As New CRMResponse
    response = conn.Send(request, 60)
    Dim rs As CRMResultSet
    Set rs = response.ResultSetByID(transaction.ID)
    
    Set InternalSendTask = GetSyncResult(rs, MSG_ADD_TASK_SUCCESS, MSG_ERR_ADD_TASK)
    
    If (rs.IsOK) Then
        SyncItem.ID = rs.Items(1)("id")
        SyncItem.Save
    End If
    Exit Function
    
handleError:
    Set InternalSendTask = GetSyncResultFromError(Err, MSG_ERR_ADD_TASK)
End Function

Private Function InternalSendMail(ByVal Mail As MailItem) As SyncResult
    On Error GoTo handleError

    Dim ErrorTexts As String
    If Not IsValidMailItem(Mail, ErrorTexts) Then
        Err.Raise ERR_INVALID_ITEM, Description:=ErrorTexts
    End If

    Dim SyncItem As SyncItem
    Set SyncItem = New SyncItem
    Set SyncItem.OutlookItem = Mail
    
    Dim ID As Integer
    Dim textSuccess As String
    Dim textError As String
    Dim transaction As CRMTransaction
    If IsSentMail(Mail) Then
        textSuccess = MSG_ADD_SENT_MAIL_SUCCESS
        textError = MSG_ERR_ADD_SENT_MAIL
        Call SetCurrentContact(Mail.Recipients.Item(1).Name, Mail.Recipients.Item(1).Address)
        ID = GetContactID(Mail.Recipients.Item(1).Address)
        Set transaction = GetSentMailTransaction(Mail, ID)
    Else
        textSuccess = MSG_ADD_RECEIVED_MAIL_SUCCESS
        textError = MSG_ERR_ADD_RECEIVED_MAIL
        Call SetCurrentContact(Mail.SenderName, Mail.SenderEmailAddress)
        ID = GetContactID(Mail.SenderEmailAddress)
        Set transaction = GetReceivedMailTransaction(Mail, ID)
    End If
    transaction.Properties.Add "id"
    
    Dim request As New CRMRequest
    request.Add transaction
    
    Dim conn As New CRMConnection
    Set conn.ConnectionParams = CreateConnectionParamsWithBasicAuthentication( _
        CentricApp.Preferences.Url, _
        CentricApp.Preferences.UserName, _
        CentricApp.Preferences.Password _
    )
    
    Dim response As New CRMResponse
    response = conn.Send(request, 60)
    Dim rs As CRMResultSet
    Set rs = response.ResultSetByID(transaction.ID)
    
    Set InternalSendMail = GetSyncResult(rs, textSuccess, textError)
    
    If (rs.IsOK) Then
        SyncItem.ID = rs.Items(1)("id")
        SyncItem.Save
    End If
    Exit Function
    
handleError:
    Set InternalSendMail = GetSyncResultFromError(Err, textError)
    If Err.Number = ERR_ABORTED_BY_USER Then
        Err.Raise Err.Number, Err.Source, Err.Description
    End If
End Function

Private Function InternalSendAppointment(ByVal Appointment As AppointmentItem) As SyncResult
    On Error GoTo handleError

    Dim ErrorTexts As String
    If Not IsValidAppointmentItem(Appointment, ErrorTexts) Then
        Err.Raise ERR_INVALID_ITEM, Description:=ErrorTexts
    End If

    Dim SyncItem As SyncItem
    Set SyncItem = New SyncItem
    Set SyncItem.OutlookItem = Appointment
    
    Dim ID As Integer
    ID = CentricApp.UserID
    
    Dim OwnerID As Integer
    OwnerID = GetAppointmentOwner(Appointment)
    
    Dim op As New CRMOperation
    op.Action = opInsert
    op.Target = "call"
    op.ShareKey = True
    With op.Conditions
        .Add "followupLength", DateDiff("n", Appointment.Start, Appointment.End)
        .Add "followupLengthDuration", 1
        .Add "alertDate", Format(Appointment.Start, "yyyy-mm-dd HH:MM:SS")
        .Add "followupEndDate", Format(Appointment.End, "yyyy-mm-dd HH:MM:SS")
        .Add "followupContactId", OwnerID
        .Add "alertText", Appointment.Subject
        .Add "alertCallTypeId", 1
        .Add "alertDateTimeZone", CentricApp.UserTimezone
        .Add "followupEndDateTimeZone", CentricApp.UserTimezone
        .Add "followupNotes", Appointment.Body
        .Add "priorityId", 2
        .Add "statusId", 3
        .Add "followupLocation", Appointment.Location
        .Add "reminder", Appointment.ReminderSet
        .Add "emailFollowupParticipants", False
        .Add "enteredBy", ID
        .Add "modifiedBy", ID
        .Add "contactId", OwnerID
        .Add "owner", ID
    End With

    Dim transaction As New CRMTransaction
    transaction.Properties.Add "id"
    transaction.Add op
    
    AddAppointmentRecipients transaction, Appointment
    AddAppointmentContacts transaction, Appointment
    
    Dim request As New CRMRequest
    request.Add transaction
    
    Dim conn As New CRMConnection
    Set conn.ConnectionParams = CreateConnectionParamsWithBasicAuthentication( _
        CentricApp.Preferences.Url, _
        CentricApp.Preferences.UserName, _
        CentricApp.Preferences.Password _
    )
    
    Dim response As New CRMResponse
    response = conn.Send(request, 60)
    Dim rs As CRMResultSet
    Set rs = response.ResultSetByID(transaction.ID)
    
    Set InternalSendAppointment = GetSyncResult(rs, MSG_ADD_APPOINTMENT_SUCCESS, MSG_ERR_ADD_APPOINTMENT)
    
    If (rs.IsOK) Then
        SyncItem.ID = rs.Items(1)("id")
        SyncItem.Save
    End If
    
    Exit Function
    
handleError:
    Set InternalSendAppointment = GetSyncResultFromError(Err, MSG_ERR_ADD_APPOINTMENT)
    If Err.Number = ERR_ABORTED_BY_USER Then
        Err.Raise Err.Number, Err.Source, Err.Description
    End If
End Function


Private Function GetSentMailTransaction(ByVal Mail As MailItem, ByVal RecepientID As Integer) As CRMTransaction
    Dim transaction As New CRMTransaction
    Dim op As CRMOperation
    Dim UserID As Integer
    UserID = CentricApp.UserID
    
    Set op = New CRMOperation
    op.Action = opInsert
    op.ShareKey = True
    op.Target = "instantCampaign"
    With op.Conditions
        .Add "name", Mail.Subject
        .Add "description", Mail.Subject
        .Add "messageID", -1
        .Add "replyTo", Mail.Recipients(1).Address
        .Add "subject", Mail.Subject
        .Add "message", Mail.Body
        .Add "type", 2
        .Add "statusId", 4
        .Add "active", "true"
        .Add "activeDate", Format(Mail.SentOn, "yyyy-mm-dd HH:MM:SS") ' FOR TEST PROPOSES Format(DateAdd("d", 1, Now), "yyyy-mm-dd HH:MM:SS")
        .Add "sendMethodId", 1
        .Add "approvedBy", UserID
        .Add "enabled", 1
        .Add "enteredBy", UserID
        .Add "modifiedBy", UserID
    End With
  
    transaction.Add op
    
    Dim sentDate As String
    sentDate = Format(Mail.SentOn, "yyyy-mm-dd HH:MM:SS")
    
    Set op = New CRMOperation
    op.Action = opInsert
    op.Target = "scheduledRecipient"
    With op.Conditions
        .Add "campaignId", "$C{instantCampaign.id}"
        .Add "contactId", RecepientID
        .Add "runId", 0
        .Add "statusId", 0
        .Add "status", ""
        '.Add "statusDate", ""
        '.Add "scheduledDate", ""
        .Add "sentDate", sentDate ' Format(CDate(Mail.SentOn), "yyyy-mm-dd")
        .Add "replyDate", sentDate
        .Add "bounceDate", sentDate
    End With
  
    transaction.Add op
  
    Set GetSentMailTransaction = transaction
End Function

Private Function GetSentMailUpdateTransaction(ByVal SyncItem As SyncItem) As CRMTransaction
    Dim Mail As MailItem
    Set Mail = SyncItem.OutlookItem

    Dim transaction As New CRMTransaction
    Dim op As CRMOperation
    Dim UserID As Integer
    UserID = CentricApp.UserID
    
    Dim timestamp As String
    timestamp = GetTimestamp("campaignList", SyncItem.ID)
    
    Set op = New CRMOperation
    op.Action = opUpdate
    op.Target = "instantCampaign"
    With op.Conditions
        .Add "id", SyncItem.ID
        .Add "name", Mail.Subject
        .Add "description", Mail.Subject
        .Add "replyTo", Mail.Recipients(1).Address
        .Add "subject", Mail.Subject
        .Add "message", Mail.Body
        .Add "activeDate", Format(Mail.SentOn, "yyyy-mm-dd HH:MM:SS") ' FOR TEST PROPOSES Format(DateAdd("d", 1, Now), "yyyy-mm-dd HH:MM:SS")
        .Add "approvedBy", UserID
        .Add "enteredBy", UserID
        .Add "modifiedBy", UserID
        .Add "modified", timestamp
        .Add "enabled", 1
    End With
  
    transaction.Add op
    
    Set GetSentMailUpdateTransaction = transaction
End Function

Private Function InternalUpdateContact(ByVal contact As ContactItem) As SyncResult
    On Error GoTo handleError
    
    Dim SyncItem As SyncItem
    Set SyncItem = New SyncItem
    Set SyncItem.OutlookItem = contact

    Dim ID As Integer
    ID = CentricApp.UserID
    
    DeleteContactOptionalFields (SyncItem.ID)
    
    Dim timestamp As String
    timestamp = GetTimestamp("contactList", SyncItem.ID)
    
    Dim op As New CRMOperation
    op.Action = opUpdate
    op.Target = "contact"
    op.ShareKey = True
    With op.Conditions
        .Add "id", SyncItem.ID
        .Add "disableNameValidation", True
        .Add "company", contact.CompanyName
        .Add "orgName", contact.CompanyName
        .Add "title", contact.Title
        .Add "nameSalutation", contact.Suffix
        .Add "nameFirst", contact.FirstName
        .Add "nameMiddle", contact.MiddleName
        .Add "nameLast", contact.LastName
        .Add "nameSuffix", ""
        .Add "notes", contact.Body
        .Add "accessType", 2
        .Add "enteredBy", ID
        .Add "modified", timestamp
        .Add "owner", ID
    End With
    
    Dim transaction As New CRMTransaction
    transaction.Add op
    AddContactOptionalFields transaction, contact, True
    
    Dim request As New CRMRequest
    request.Add transaction
    
    Dim conn As New CRMConnection
    Set conn.ConnectionParams = CreateConnectionParamsWithBasicAuthentication( _
        CentricApp.Preferences.Url, _
        CentricApp.Preferences.UserName, _
        CentricApp.Preferences.Password _
    )
    
    Dim response As New CRMResponse
    response = conn.Send(request, 60)
    
    Dim rs As CRMResultSet
    Set rs = response.ResultSetByID(transaction.ID)
    
    Set InternalUpdateContact = GetSyncResult(rs, MSG_UPDATE_CONTACT_SUCCESS, MSG_ERR_UPDATE_CONTACT)
    
    If rs.IsOK Then SyncItem.Save
    Exit Function
    
handleError:
    If Err.Number = ERR_ITEM_NOT_FOUND Then
        Set InternalUpdateContact = InternalSendContact(contact)
    Else
        Set InternalUpdateContact = GetSyncResultFromError(Err, MSG_ERR_UPDATE_CONTACT)
    End If
End Function

Private Function InternalUpdateTask(ByVal Task As TaskItem) As SyncResult
    On Error GoTo handleError
    
    Dim SyncItem As SyncItem
    Set SyncItem = New SyncItem
    Set SyncItem.OutlookItem = Task
    
    Dim ID As Integer
    ID = CentricApp.UserID
    
    Dim timestamp As String
    timestamp = GetTimestamp("taskList", SyncItem.ID)
    
    Dim op As New CRMOperation
    op.Action = opUpdate
    op.Target = "task"
    With op.Conditions
        .Add "id", SyncItem.ID
        .Add "description", Task.Subject
        .Add "notes", Task.Body
        ' TODO: Use Outlook task priority instead of hardcoded value
        .Add "priority", 5
        .Add "enteredBy", ID
        .Add "modifiedBy", ID
        .Add "owner", ID
        .Add "complete", Task.Complete
        .Add "enabled", 1
        .Add "dueDate", DateAsString(Task.DueDate)
        .Add "completeDate", DateAsString(Task.DateCompleted)
        .Add "entered", 0
        .Add "modified", timestamp
    End With

    Dim transaction As New CRMTransaction
    transaction.Add op
    Dim request As New CRMRequest
    request.Add transaction
    
    Dim conn As New CRMConnection
    Set conn.ConnectionParams = CreateConnectionParamsWithBasicAuthentication( _
        CentricApp.Preferences.Url, _
        CentricApp.Preferences.UserName, _
        CentricApp.Preferences.Password _
    )
    
    Dim response As New CRMResponse
    response = conn.Send(request, 60)
    
    Dim rs As CRMResultSet
    Set rs = response.ResultSetByID(transaction.ID)
    Set InternalUpdateTask = GetSyncResult(rs, MSG_UPDATE_TASK_SUCCESS, MSG_ERR_UPDATE_TASK)
    
    If rs.IsOK Then SyncItem.Save

    Exit Function
    
handleError:
    If Err.Number = ERR_ITEM_NOT_FOUND Then
        Set InternalUpdateTask = InternalSendTask(Task)
    Else
        Set InternalUpdateTask = GetSyncResultFromError(Err, MSG_ERR_UPDATE_TASK)
    End If
End Function

Private Function InternalUpdateMail(ByVal Mail As MailItem) As SyncResult
    On Error GoTo handleError
    
    Dim SyncItem As SyncItem
    Set SyncItem = New SyncItem
    Set SyncItem.OutlookItem = Mail

    Dim transaction As CRMTransaction
    Dim textSuccess As String
    Dim textError As String
    If IsSentMail(Mail) Then
        textSuccess = MSG_UPDATE_SENT_MAIL_SUCCESS
        textError = MSG_ERR_UPDATE_SENT_MAIL
        Set transaction = GetSentMailUpdateTransaction(SyncItem)
    Else
        textSuccess = MSG_UPDATE_RECEIVED_MAIL_SUCCESS
        textError = MSG_ERR_UPDATE_RECEIVED_MAIL
        Set transaction = GetReceivedMailUpdateTransaction(SyncItem)
    End If
    
    Dim request As New CRMRequest
    request.Add transaction
    
    Dim conn As New CRMConnection
    Set conn.ConnectionParams = CreateConnectionParamsWithBasicAuthentication( _
        CentricApp.Preferences.Url, _
        CentricApp.Preferences.UserName, _
        CentricApp.Preferences.Password _
    )
    
    Dim response As New CRMResponse
    response = conn.Send(request, 60)
    Dim rs As CRMResultSet
    Set rs = response.ResultSetByID(transaction.ID)
    
    Set InternalUpdateMail = GetSyncResult(rs, textSuccess, textError)
    
    If rs.IsOK Then SyncItem.Save
    
    Exit Function
    
handleError:
    If Err.Number = ERR_ITEM_NOT_FOUND Then
        Set InternalUpdateMail = InternalSendMail(Mail)
    Else
        Set InternalUpdateMail = GetSyncResultFromError(Err, textError)
        If Err.Number = ERR_ABORTED_BY_USER Then
            Err.Raise Err.Number, Err.Source, Err.Description
        End If
    End If
End Function

Private Function InternalUpdateAppointment(ByVal Appointment As AppointmentItem) As SyncResult
    On Error GoTo handleError
    
    Dim SyncItem As SyncItem
    Set SyncItem = New SyncItem
    Set SyncItem.OutlookItem = Appointment
    
    Dim ID As Integer
    ID = CentricApp.UserID
    
    Dim OwnerID As Integer
    OwnerID = GetAppointmentOwner(Appointment)
    
    Dim timestamp As String
    timestamp = GetTimestamp("callList", SyncItem.ID)
    
    Dim op As New CRMOperation
    op.Action = opUpdate
    op.Target = "call"
    With op.Conditions
        .Add "id", SyncItem.ID
        .Add "followupLength", DateDiff("n", Appointment.Start, Appointment.End)
        .Add "followupLengthDuration", 1
        .Add "alertDate", Format(Appointment.Start, "yyyy-mm-dd HH:MM:SS")
        .Add "followupEndDate", Format(Appointment.End, "yyyy-mm-dd HH:MM:SS")
        .Add "followupContactId", OwnerID
        .Add "alertText", Appointment.Subject
        .Add "alertCallTypeId", 1
        .Add "followupNotes", Appointment.Body
        .Add "followupLocation", Appointment.Location
        .Add "alertDateTimeZone", CentricApp.UserTimezone
        .Add "followupEndDateTimeZone", CentricApp.UserTimezone
        .Add "priorityId", 2
        .Add "statusId", 3
        .Add "reminder", Appointment.ReminderSet
        .Add "emailFollowupParticipants", False
        .Add "modifiedBy", ID
        .Add "modified", timestamp
        .Add "contactId", OwnerID
        .Add "owner", ID
    End With

    Dim transaction As New CRMTransaction
    transaction.Add op
    
    Dim request As New CRMRequest
    request.Add transaction
    
    Dim conn As New CRMConnection
    Set conn.ConnectionParams = CreateConnectionParamsWithBasicAuthentication( _
        CentricApp.Preferences.Url, _
        CentricApp.Preferences.UserName, _
        CentricApp.Preferences.Password _
    )
    
    Dim response As New CRMResponse
    response = conn.Send(request, 60)
    
    Dim rs As CRMResultSet
    Set rs = response.ResultSetByID(transaction.ID)
    
    Set InternalUpdateAppointment = GetSyncResult(rs, MSG_UPDATE_APPOINTMENT_SUCCESS, MSG_ERR_UPDATE_APPOINTMENT)
    
    Set transaction = New CRMTransaction
    Set op = New CRMOperation
    op.Action = opDelete
    op.Target = "callParticipant"
    op.Conditions.Add "callId", SyncItem.ID
    transaction.Add op
    Set request = New CRMRequest
    request.Add transaction
    response = conn.Send(request, 60)
    
    Set transaction = New CRMTransaction
    AddAppointmentRecipients transaction, Appointment, SyncItem.ID
    AddAppointmentContacts transaction, Appointment, SyncItem.ID

    Set request = New CRMRequest
    request.Add transaction
    response = conn.Send(request, 60)
    
    If rs.IsOK Then SyncItem.Save
    
    Exit Function
    
handleError:
    If Err.Number = ERR_ITEM_NOT_FOUND Then
        Set InternalUpdateAppointment = InternalSendAppointment(Appointment)
    Else
        Set InternalUpdateAppointment = GetSyncResultFromError(Err, MSG_ERR_UPDATE_APPOINTMENT)
        If Err.Number = ERR_ABORTED_BY_USER Then
            Err.Raise Err.Number, Err.Source, Err.Description
        End If
    End If
End Function

Private Function GetReceivedMailTransaction(ByVal Mail As MailItem, ByVal SenderID As Integer) As CRMTransaction
    Dim transaction As New CRMTransaction
    Dim op As CRMOperation
    Dim UserID As Integer
    UserID = CentricApp.UserID
    
    Set op = New CRMOperation
    op.Action = opInsert
    op.ShareKey = True
    op.Target = "message"
    With op.Conditions
        .Add "name", Mail.Subject
        .Add "description", Mail.Subject
        .Add "messageSubject", Mail.Subject
        .Add "messageText", Mail.Body
        .Add "replyTo", Mail.SenderEmailAddress
        .Add "enabled", "true"
        .Add "accessType", 1
        .Add "enteredBy", UserID
        .Add "modifiedBy", UserID
    End With
    transaction.Add op
    
    Set op = New CRMOperation
    op.Action = opInsert
    op.Target = "contactMessage"
    With op.Conditions
        .Add "messageId", "$C{message.id}"
        .Add "receivedDate", Format(Now, "yyyy-mm-dd HH:MM:SS")
        .Add "receivedFrom", SenderID
        .Add "receivedBy", UserID
    End With
    transaction.Add op
    
    Set GetReceivedMailTransaction = transaction
End Function

Private Function GetReceivedMailUpdateTransaction(ByVal SyncItem As SyncItem) As CRMTransaction
    Dim Mail As MailItem
    Set Mail = SyncItem.OutlookItem
    
    Dim transaction As New CRMTransaction
    Dim op As CRMOperation
    Dim UserID As Integer
    UserID = CentricApp.UserID
    
    Dim timestamp As String
    timestamp = GetTimestamp("messageList", SyncItem.ID)
    
    Set op = New CRMOperation
    op.Action = opUpdate
    op.Target = "message"
    With op.Conditions
        .Add "id", SyncItem.ID
        .Add "name", Mail.Subject
        .Add "description", Mail.Subject
        .Add "messageSubject", Mail.Subject
        .Add "messageText", Mail.Body
        .Add "enteredBy", UserID
        .Add "modifiedBy", UserID
        .Add "modified", timestamp
        .Add "enabled", "true"
        .Add "accessType", 1
        .Add "disableReplyToValidation", "true"
    End With
    transaction.Add op
    
    Set GetReceivedMailUpdateTransaction = transaction
End Function

Public Sub ShowOptionsCentric()
  frmPreferences.Show vbModal
End Sub

Private Sub EnableMenu()
  CentricApp.EnableBar = True
End Sub

Private Sub DisableMenu()
  CentricApp.EnableBar = False
End Sub

Private Function CheckForPreferences() As Boolean
  If CentricApp.Preferences.UserName = "" Or CentricApp.Preferences.Password = "" Or CentricApp.Preferences.Url = "" Then
    CheckForPreferences = False
    'frmprefer.Show
    ShowOptionsCentric
    Exit Function
  End If
  
  CheckForPreferences = True
End Function

Private Sub LogSyncResult(ByVal SyncResult As SyncResult)
    Dim msg As String
    msg = SyncResult.Description
    If Not SyncResult.IsOK Then
        msg = msg & " ERROR#" & SyncResult.Code
    End If
    If SyncResult.Details <> "" Then
        msg = msg & " DETAILS:" & SyncResult.Details
    End If
    CentricApp.Logger.Debugging msg
End Sub

Public Sub DisplaySyncResult(ByVal SyncResult As SyncResult, ByVal Title As String)
    If SyncResult.Code = ERR_ABORTED_BY_USER Then
        MsgBox MSG_ERR_ABORTED_BY_USER, vbExclamation + vbOKOnly, Title
        Exit Sub
    End If
    
    Dim style As VbMsgBoxStyle
    If (SyncResult.IsOK) Then
        style = vbInformation
    Else
        style = vbExclamation
    End If
    Dim msg As String
    msg = SyncResult.Description
    'If Not SyncResult.IsOK Then
    '    msg = msg & vbCrLf & " ERROR#" & SyncResult.Code
    'End If
    If SyncResult.Details <> "" Then
        msg = msg & vbCrLf & SyncResult.Details
    End If
    MsgBox msg, style + vbOKOnly, Title
End Sub

Public Function GetSyncResult(ByVal rs As CRMResultSet, ByVal SuccessText As String, ByVal ErrorText As String) As SyncResult
    Dim SyncResult As New SyncResult
    SyncResult.Code = rs.ErrorCode
    SyncResult.Details = rs.ErrorText
    If SyncResult.IsOK Then
        SyncResult.Description = SuccessText
    Else
        SyncResult.Description = ErrorText
    End If
    Set GetSyncResult = SyncResult
End Function

Public Function GetSyncResultFromError(ByVal Error As ErrObject, ByVal text As String) As SyncResult
    Dim SyncResult As New SyncResult
    SyncResult.Code = Error.Number
    SyncResult.Details = Error.Description
    SyncResult.Description = text
    Set GetSyncResultFromError = SyncResult
End Function

Private Sub AddContactOptionalFields(ByVal transaction As CRMTransaction, ByVal contact As ContactItem, ByVal ForUpdate As Boolean)
    Dim op As CRMOperation
    If ForUpdate Then
        Dim SyncItem As SyncItem
        Set SyncItem = New SyncItem
        Set SyncItem.OutlookItem = contact

        'check for business address
        If Len(contact.BusinessAddress) > 0 Then
            Set op = GetContactAddressOperation( _
                contact.BusinessAddressStreet, contact.BusinessAddressCity, _
                contact.BusinessAddressState, contact.BusinessAddressPostalCode, _
                contact.BusinessAddressCountry, SyncItem.ID _
            )
            transaction.Add op
        End If
        
        'check for home address
        If Len(contact.HomeAddress) > 0 Then
            Set op = GetContactAddressOperation( _
                contact.HomeAddressStreet, contact.HomeAddressCity, _
                contact.HomeAddressState, contact.HomeAddressPostalCode, _
                contact.HomeAddressCountry, SyncItem.ID _
            )
            transaction.Add op
        End If
        
        'check for other address
        If Len(contact.OtherAddress) > 0 Then
            Set op = GetContactAddressOperation( _
                contact.OtherAddressStreet, contact.OtherAddressCity, _
                contact.OtherAddressState, contact.OtherAddressPostalCode, _
                contact.OtherAddressCountry, SyncItem.ID _
            )
            transaction.Add op
        End If
        
        ' check for business email
        If Len(contact.Email1Address) > 0 Then
            transaction.Add GetContactEmailOperation(contact.Email1Address, SyncItem.ID)
        End If
        
        ' check for home email
        If Len(contact.Email2Address) > 0 Then
            transaction.Add GetContactEmailOperation(contact.Email2Address, SyncItem.ID)
        End If
        
        ' check for other email
        If Len(contact.Email3Address) > 0 Then
            transaction.Add GetContactEmailOperation(contact.Email3Address, SyncItem.ID)
        End If
        
        'check for business phone
        If Len(contact.BusinessTelephoneNumber) > 0 Then
            transaction.Add GetContactPhoneOperation(contact.BusinessTelephoneNumber, ptBusiness, SyncItem.ID)
        End If
        
        If Len(contact.Business2TelephoneNumber) > 0 Then
            transaction.Add GetContactPhoneOperation(contact.Business2TelephoneNumber, ptBusiness2, SyncItem.ID)
        End If
        If Len(contact.BusinessFaxNumber) > 0 Then
            transaction.Add GetContactPhoneOperation(contact.BusinessFaxNumber, ptBusinessFax, SyncItem.ID)
        End If
        
        'check for home phone
        If Len(contact.HomeTelephoneNumber) > 0 Then
            transaction.Add GetContactPhoneOperation(contact.HomeTelephoneNumber, ptHome, SyncItem.ID)
        End If
        If Len(contact.Home2TelephoneNumber) > 0 Then
            transaction.Add GetContactPhoneOperation(contact.Home2TelephoneNumber, ptHome2, SyncItem.ID)
        End If
        If Len(contact.HomeFaxNumber) > 0 Then
            transaction.Add GetContactPhoneOperation(contact.HomeFaxNumber, ptHomeFax, SyncItem.ID)
        End If
        
        'check for mobile phone
        If Len(contact.MobileTelephoneNumber) > 0 Then
            transaction.Add GetContactPhoneOperation(contact.MobileTelephoneNumber, ptMobile, SyncItem.ID)
        End If
        
        'check for pager phone
        If Len(contact.PagerNumber) > 0 Then
            transaction.Add GetContactPhoneOperation(contact.PagerNumber, ptPager, SyncItem.ID)
        End If
        
        'check for other phones
        If Len(contact.OtherTelephoneNumber) > 0 Then
            transaction.Add GetContactPhoneOperation(contact.OtherTelephoneNumber, ptOther, SyncItem.ID)
        End If
    Else
        'check for business address
        If Len(contact.BusinessAddress) > 0 Then
            Set op = GetContactAddressOperation( _
                contact.BusinessAddressStreet, contact.BusinessAddressCity, _
                contact.BusinessAddressState, contact.BusinessAddressPostalCode, _
                contact.BusinessAddressCountry _
            )
            transaction.Add op
        End If
        
        'check for home address
        If Len(contact.HomeAddress) > 0 Then
            Set op = GetContactAddressOperation( _
                contact.HomeAddressStreet, contact.HomeAddressCity, _
                contact.HomeAddressState, contact.HomeAddressPostalCode, _
                contact.HomeAddressCountry _
            )
            transaction.Add op
        End If
        
        'check for other address
        If Len(contact.OtherAddress) > 0 Then
            Set op = GetContactAddressOperation( _
                contact.OtherAddressStreet, contact.OtherAddressCity, _
                contact.OtherAddressState, contact.OtherAddressPostalCode, _
                contact.OtherAddressCountry _
            )
            transaction.Add op
        End If
        
        ' check for business email
        If Len(contact.Email1Address) > 0 Then
            transaction.Add GetContactEmailOperation(contact.Email1Address)
        End If
        
        ' check for home email
        If Len(contact.Email2Address) > 0 Then
            transaction.Add GetContactEmailOperation(contact.Email2Address)
        End If
        
        ' check for other email
        If Len(contact.Email3Address) > 0 Then
            transaction.Add GetContactEmailOperation(contact.Email3Address)
        End If
        
        'check for business phone
        If Len(contact.BusinessTelephoneNumber) > 0 Then
            transaction.Add GetContactPhoneOperation(contact.BusinessTelephoneNumber, ptBusiness)
        End If
        If Len(contact.Business2TelephoneNumber) > 0 Then
            transaction.Add GetContactPhoneOperation(contact.Business2TelephoneNumber, ptBusiness2)
        End If
        If Len(contact.BusinessFaxNumber) > 0 Then
            transaction.Add GetContactPhoneOperation(contact.BusinessFaxNumber, ptBusinessFax)
        End If
        
        'check for home phone
        If Len(contact.HomeTelephoneNumber) > 0 Then
            transaction.Add GetContactPhoneOperation(contact.HomeTelephoneNumber, ptHome)
        End If
        If Len(contact.Home2TelephoneNumber) > 0 Then
            transaction.Add GetContactPhoneOperation(contact.Home2TelephoneNumber, ptHome2)
        End If
        If Len(contact.HomeFaxNumber) > 0 Then
            transaction.Add GetContactPhoneOperation(contact.HomeFaxNumber, ptHomeFax)
        End If
        
        'check for mobile phone
        If Len(contact.MobileTelephoneNumber) > 0 Then
            transaction.Add GetContactPhoneOperation(contact.MobileTelephoneNumber, ptMobile)
        End If
        
        'check for pager phone
        If Len(contact.PagerNumber) > 0 Then
            transaction.Add GetContactPhoneOperation(contact.PagerNumber, ptPager)
        End If
        
        'check for other phones
        If Len(contact.OtherTelephoneNumber) > 0 Then
            transaction.Add GetContactPhoneOperation(contact.OtherTelephoneNumber, ptOther)
        End If
    End If
End Sub

Private Sub DeleteContactOptionalFields(ByVal ContactID As String)
    Dim request As New CRMRequest
    
    Dim transaction As CRMTransaction
    Dim op As CRMOperation
    
    Set transaction = New CRMTransaction
    transaction.Properties.Add "contactId"
    Set op = New CRMOperation
    op.Target = "contactPhoneNumber"
    op.Action = opDelete
    op.Conditions.Add "contactId", ContactID
    transaction.Add op
    request.Add transaction
    
    Set transaction = New CRMTransaction
    transaction.Properties.Add "contactId"
    Set op = New CRMOperation
    op.Target = "contactEmailAddress"
    op.Action = opDelete
    op.Conditions.Add "contactId", ContactID
    transaction.Add op
    request.Add transaction
    
    Set transaction = New CRMTransaction
    transaction.Properties.Add "contactId"
    Set op = New CRMOperation
    op.Target = "contactAddress"
    op.Action = opDelete
    op.Conditions.Add "contactId", ContactID
    transaction.Add op
    request.Add transaction
    
    Dim conn As New CRMConnection
    Set conn.ConnectionParams = CreateConnectionParamsWithBasicAuthentication( _
        CentricApp.Preferences.Url, _
        CentricApp.Preferences.UserName, _
        CentricApp.Preferences.Password _
    )
    
    Dim response As New CRMResponse
    response = conn.Send(request, 60)
End Sub

Private Sub AddAppointmentRecipients(ByVal transaction As CRMTransaction, ByVal Appointment As AppointmentItem, Optional CallID As Variant)
    Dim ID As Integer
    ID = CentricApp.UserID
    Dim Recipient As Recipient
    For Each Recipient In Appointment.Recipients
        Call AddSingleAppointmentRecipient(transaction, Recipient, ID, CallID)
    Next Recipient
End Sub

Private Sub AddSingleAppointmentRecipient(ByVal transaction As CRMTransaction, ByVal Recipient As Recipient, ByVal UserID As Integer, Optional CallID As Variant)
    On Error GoTo handleError
    
    Dim op As CRMOperation
    Set op = New CRMOperation
    
    op.Action = opInsert
    op.Target = "callParticipant"
    With op.Conditions
        If IsMissing(CallID) Then
            .Add "callId", "$C{call.id}"
        Else
            .Add "callId", CallID
        End If
        Call SetCurrentContact(Recipient.Name, Recipient.Address)
        .Add "contactId", GetContactID(Recipient.Address)
        .Add "isAvailable", 1
        .Add "isFollowup", 1
        .Add "enteredBy", UserID
        .Add "modifiedBy", UserID
    End With
    transaction.Add op
    Exit Sub
    
handleError:
    If Err.Number = ERR_IGNORE_CONTACT Then
        Exit Sub
    Else
        Err.Raise Err.Number, Err.Source, Err.Description
    End If
End Sub

Private Sub AddAppointmentContacts(ByVal transaction As CRMTransaction, ByVal Appointment As AppointmentItem, Optional CallID As Variant)
    Dim ID As Integer
    ID = CentricApp.UserID
    Dim contact As ContactItem
    Dim Link As Link
    For Each Link In Appointment.Links
        Set contact = Link.Item
        Call AddSingleAppointmentContact(transaction, contact, ID, CallID)
    Next Link
End Sub

Private Sub AddSingleAppointmentContact(ByVal transaction As CRMTransaction, ByVal contact As ContactItem, ByVal UserID As Integer, Optional CallID As Variant)
    On Error GoTo handleError
    
    Dim SyncItem As SyncItem
    Set SyncItem = New SyncItem
    Set SyncItem.OutlookItem = contact
    If Not SyncItem.IsSynchronized Then
        Dim SyncResult As SyncResult
        Set SyncResult = ProcessSyncItem(SyncItem)
        If Not SyncResult.IsOK Then Err.Raise SyncResult.Code, Description:=SyncResult.Details
    End If
    
    Dim op As CRMOperation
    Set op = New CRMOperation
    
    op.Action = opInsert
    op.Target = "callParticipant"
    With op.Conditions
        If IsMissing(CallID) Then
            .Add "callId", "$C{call.id}"
        Else
            .Add "callId", CallID
        End If
        .Add "contactId", SyncItem.ID
        .Add "isAvailable", 1
        .Add "isFollowup", 1
        .Add "enteredBy", UserID
        .Add "modifiedBy", UserID
    End With
    transaction.Add op
    Exit Sub
    
handleError:
    If Err.Number = ERR_IGNORE_CONTACT Then
        Exit Sub
    Else
        Err.Raise Err.Number, Err.Source, Err.Description
    End If
End Sub

Private Function GetAppointmentOwner(ByVal Appointment As AppointmentItem) As Integer
    If Appointment.Links.Count > 0 Then
        Dim SyncItem As SyncItem
        Set SyncItem = New SyncItem
        Set SyncItem.OutlookItem = Appointment.Links(1).Item
        If Not SyncItem.IsSynchronized Then
            Dim SyncResult As SyncResult
            Set SyncResult = ProcessSyncItem(SyncItem)
            If Not SyncResult.IsOK Then Err.Raise SyncResult.Code, Description:=SyncResult.Details
        End If
        
        GetAppointmentOwner = SyncItem.ID
        Exit Function
    End If
    
    Dim Recipient As Recipient
    For Each Recipient In Appointment.Recipients
        If Recipient.Type = olRequired Then
            Call SetCurrentContact(Recipient.Name, Recipient.Address)
            GetAppointmentOwner = GetContactID(Recipient.Address)
            If CentricApp.ContactID <> GetAppointmentOwner Then Exit Function
        End If
    Next Recipient
    
    GetAppointmentOwner = CentricApp.UserID
End Function
