VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Search"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
'Concourse Suite Outlook-Plugin
'Copyright (C) 2000-2007 Concursive Corporation.
'
'This program is free software; you can redistribute it and/or
'modify it under the terms of the GNU General Public License
'as published by the Free Software Foundation; version 2
'of the License.
'
'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'GNU General Public License for more details.
'
'You should have received a copy of the GNU General Public License
'along with this program; if not, write to the Free Software
'Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 

Option Explicit
Option Base 0

Private m_conditions As SearchConditions
Private m_ignoreFolders As FolderList
Private m_started As Boolean
Private m_cancel As Boolean
Private WithEvents m_app As Outlook.Application
Attribute m_app.VB_VarHelpID = -1
Private m_conditionProcessed As Boolean

Public Event Started(ByVal Search As Search)
Public Event Finished(ByVal Search As Search)
Public Event Cancelled(ByVal Search As Search)
Public Event ItemFound(ByVal Search As Search, ByVal Item As Object)
Public Event LocationSwitch(ByVal Location As String)

Public Property Get Conditions() As SearchConditions
    Set Conditions = m_conditions
End Property

Public Property Get IgnoreFolders() As FolderList
    Set IgnoreFolders = m_ignoreFolders
End Property

Public Sub Start()
    If (m_started) Then Err.Raise ERR_INVALID_STATE, Description:=MSG_ERR_SEARCH_STARTED
        
    CentricApp.Logger.Debugging "Starting Search..."
    m_started = True
    m_cancel = False
    RaiseEvent Started(Me)
    
    Dim outlookSearch As Outlook.Search
    Dim Condition As SearchCondition
    For Each Condition In Conditions
        CentricApp.Logger.Debugging "Processing search condition: " & Condition.Location & ", " & Condition.Filter
        DoEvents
        If m_cancel Then
            CentricApp.Logger.Debugging "Cancelling Search..."
            Exit For
        End If
        
        ProcessCondition Condition
    Next Condition
        
    m_started = False
    If m_cancel Then
        CentricApp.Logger.Debugging "Search cancelled."
        RaiseEvent Cancelled(Me)
    Else
        CentricApp.Logger.Debugging "Search complete."
        RaiseEvent Finished(Me)
    End If
End Sub

Public Sub Cancel()
    If (Not m_started) Then Err.Raise ERR_INVALID_STATE, Description:=MSG_ERR_SEARCH_NOT_STARTED
    
    CentricApp.Logger.Debugging "Cancelling search..."
    m_cancel = True
    RaiseEvent Cancelled(Me)
End Sub

Private Sub Class_Initialize()
    Set m_app = CentricApp.Outlook
    Set m_conditions = New SearchConditions
    Set m_ignoreFolders = New FolderList
End Sub

Private Sub Class_Terminate()
    Set m_ignoreFolders = Nothing
    Set m_conditions = Nothing
    Set m_app = Nothing
End Sub

Private Sub ProcessCondition(ByVal Condition As SearchCondition)
    ProcessFolder Condition.Location, Condition.Filter, Condition.SearchSubFolders
End Sub

Private Sub ProcessFolder(ByVal Folder As String, ByVal Filter As String, ByVal SearchSubFolders As Boolean)
    If (m_ignoreFolders.Contains(Folder)) Then Exit Sub
    
    Dim m_SearchObject As Outlook.Search
    m_conditionProcessed = False
    CentricApp.Logger.Debugging "Processing folder: " & Folder & ", " & Filter
    RaiseEvent LocationSwitch(Folder)
    Set m_SearchObject = m_app.AdvancedSearch("'" & Folder & "'", Filter, False, ObjPtr(Me))
    
    Dim Results As Outlook.Results
    Dim Item As Object
    
    On Error GoTo CondittionError
    
    Set Results = m_SearchObject.Results
    'Call Results.SetColumns("LastModificationTime")  ' , " & PROP_CENTRIC_ITEM_ID & ", " & PROP_LAST_SYNCHRONIZED)
    Dim Count As Integer
    Count = 0
    
    Do
        If (Count = 0) Then
            Set Item = Results.GetFirst
        Else
            Set Item = Results.GetNext
        End If
        If Item Is Nothing Then
            If m_conditionProcessed Then
                Exit Do
            Else
                DoEvents
            End If
        Else
            CentricApp.Logger.Debugging "Item found: " & Item
            RaiseEvent ItemFound(Me, Item)
            Count = Count + 1
        End If
    Loop
    Debug.Assert Count = Results.Count
    CentricApp.Logger.Debugging "Folder processed (" & Count & " items found)."
    
CondittionError:
        If (Err.Number <> 0) Then CentricApp.Logger.Error "Error while processing condition (" & _
        Err.Number & " " & Err.Description & ")."
    
    DoEvents
    
    If m_cancel Then Exit Sub
    If Not SearchSubFolders Then Exit Sub
    
    Dim currentFolder As MAPIFolder
    Set currentFolder = GetMAPIFolder(Folder)
    Dim children As Folders
    Set children = currentFolder.Folders
    Set currentFolder = children.GetFirst
    Do While Not currentFolder Is Nothing
        ProcessFolder currentFolder.folderPath, Filter, True
        Set currentFolder = children.GetNext
    Loop
End Sub

Private Sub m_app_AdvancedSearchComplete(ByVal SearchObject As Outlook.Search)
    If ObjPtr(Me) <> SearchObject.Tag Then Exit Sub
    CentricApp.Logger.Debugging "Outlook finished condition search..."
    m_conditionProcessed = True
End Sub

Private Sub m_app_AdvancedSearchStopped(ByVal SearchObject As Outlook.Search)
    m_conditionProcessed = True
End Sub
