Attribute VB_Name = "ShellUtils"
'Concourse Suite Outlook-Plugin
'Copyright (C) 2000-2007 Concursive Corporation.
'
'This program is free software; you can redistribute it and/or
'modify it under the terms of the GNU General Public License
'as published by the Free Software Foundation; version 2
'of the License.
'
'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'GNU General Public License for more details.
'
'You should have received a copy of the GNU General Public License
'along with this program; if not, write to the Free Software
'Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

Option Explicit

' Concourse Suite plugin data folder (under %APPDATA% path)
Private Const FOLDER_CENTRIC_CRM = "Concourse Suite Outlook Plugin"

' Display options for ShellExecute function
Private Const SW_SHOWNORMAL = 1

' Operation verbs for ShellExecute function
Private Const VERB_OPEN = "open"

' ShellExecute helper function
'
' Performs an operation on a specified file, folder ot URL
'
' hwnd - Owner window handler. 0 if no parent window
' lpOperation - Verb of action to be executed (like "open", "explore" etc)
' lpFile - File or folder name, or URL
' lpParameters - Additional parameters for the command
' lpDirectory - Default directory
' nShowCmd - One of predefined window display options (like SW_SHOWNORMAL)
Private Declare Function ShellExecute Lib "shell32.dll" Alias "ShellExecuteA" _
(ByVal hwnd As Long, _
ByVal lpOperation As String, _
ByVal lpFile As String, _
ByVal lpParameters As String, _
ByVal lpDirectory As String, _
ByVal nShowCmd As Long) As Long


' OpenUrlInBrowser
'
' Opens specified URL in default system browser
'
Public Sub OpenUrlInBrowser(ByVal Url As String)
    
    ShellExecute 0, VERB_OPEN, Url, "", 0, SW_SHOWNORMAL
    
End Sub

' Returns path to plugin data folder
Public Function GetPluginDataFolder() As String
    Dim shell As New Shell32.shell
    
    On Error Resume Next
    
    Dim appDataFolder As String
    appDataFolder = shell.NameSpace(Shell32.ssfAPPDATA).Self.path
    
    GetPluginDataFolder = appDataFolder & "\" & FOLDER_CENTRIC_CRM
End Function

' Opens text file
Public Function OpenTextFile(path As String, Mode As IOMode) As TextStream
    Dim fso As New FileSystemObject
    EnsureParentFolderExists fso, path
    
    Set OpenTextFile = fso.OpenTextFile(path, Mode, True)
End Function

Private Sub EnsureParentFolderExists(fso As FileSystemObject, path As String)
    Dim parentPath As String
    parentPath = fso.GetParentFolderName(path)
    If fso.FolderExists(parentPath) Then
        Exit Sub
    End If
    
    EnsureParentFolderExists fso, parentPath
    
    fso.CreateFolder parentPath
End Sub

