Attribute VB_Name = "Sync"
'Concourse Suite Outlook-Plugin
'Copyright (C) 2000-2007 Concursive Corporation.
'
'This program is free software; you can redistribute it and/or
'modify it under the terms of the GNU General Public License
'as published by the Free Software Foundation; version 2
'of the License.
'
'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'GNU General Public License for more details.
'
'You should have received a copy of the GNU General Public License
'along with this program; if not, write to the Free Software
'Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 

Option Explicit

'Public Const PROP_SYNC_STATUS = "urn:schemas:centriccrm:syncstatus"
Public Const PROP_CENTRIC_ITEM_ID = "urn:schemas:centriccrm:itemid"
Public Const PROP_LAST_SYNCHRONIZED = "urn:schemas:centriccrm:lastsynchronized"

Public Const STATUS_UNKNOWN = -1
Public Const STATUS_NEW = 0
Public Const STATUS_SYNCHRONIZED = 1
Public Const STATUS_UPDATED = 2
'Public Const STATUS_NON_SYNCHRONIZABLE = -2

Public Const SYNC_RESULT_OK = 0
Public Const SYNC_RESULT_ERROR = 1

Public Const FOLDER_LIST_SEPARATOR = ";"
Public Const FOLDER_LIST_DEFAULT = "[default]"

Public Const INDEX_LAST_SYNC = 10


Private m_Initialized As Boolean
Private m_IntervalLabels() As String
Private m_IntervalValues() As String
Private m_BatchMode As Boolean

Private Sub initialize()
    IsBatchMode = False
    
    ReDim m_IntervalValues(0 To 10)
    ReDim m_IntervalLabels(0 To 10)

    AddInterval 0, "n 15", "15 minutes"
    AddInterval 1, "n 30", "30 minutes"
    AddInterval 2, "h 1", "1 hour"
    AddInterval 3, "h 12", "12 hours"
    AddInterval 4, "d 1", "1 day"
    AddInterval 5, "d 2", "2 days"
    AddInterval 6, "ww1", "1 week"
    AddInterval 7, "ww2", "2 weeks"
    AddInterval 8, "m 1", "1 month"
    AddInterval 9, "m 2", "2 months"
    AddInterval INDEX_LAST_SYNC, "", "Since Last Sync"
    
    m_Initialized = True
End Sub

Private Sub AddInterval(Index As Integer, value As String, Label As String)
    m_IntervalValues(Index) = value
    m_IntervalLabels(Index) = Label
End Sub

Public Property Get IsBatchMode() As Boolean
    IsBatchMode = m_BatchMode
End Property

Public Property Let IsBatchMode(Mode As Boolean)
    m_BatchMode = Mode
End Property

Public Property Get IntervalLabels() As String()
    If Not m_Initialized Then initialize
    IntervalLabels = m_IntervalLabels
End Property

Public Function GetIntervalLabel(Index As Integer) As String
    Dim labels() As String
    labels = IntervalLabels
    GetIntervalLabel = labels(Index)
End Function

Public Sub StartSync()
    If Not m_Initialized Then initialize
    frmSync.Show
End Sub

Public Function GetSelectedIntervalStart() As Date
    If Not m_Initialized Then initialize
    If (CentricApp.Preferences.SyncInterval = INDEX_LAST_SYNC) Then
        GetSelectedIntervalStart = CentricApp.Preferences.LastSync
    Else
        Dim interval As String
        interval = m_IntervalValues(CentricApp.Preferences.SyncInterval)
        Dim d As Date
        d = DateAdd(Trim$(Left$(interval, 2)), -Right(interval, Len(interval) - 2), Now)
        If CentricApp.Preferences.SyncInterval >= 4 Then ' For days and months, get the start of the day
            GetSelectedIntervalStart = DateSerial(Year(d), Month(d), Day(d))
        Else
            GetSelectedIntervalStart = d
        End If
    End If
End Function

Public Function GetDefaultSyncFolders() As String
    Dim Folder As MAPIFolder
    Set Folder = CentricApp.Outlook.Session.GetDefaultFolder(olFolderInbox)
    Do While (TypeOf Folder.parent Is MAPIFolder)
        Set Folder = Folder.parent
    Loop
    GetDefaultSyncFolders = Folder.folderPath
End Function

Public Function GetDefaultIgnoreFolders() As String
    Dim FolderList As FolderList
    Set FolderList = New FolderList
    Dim NameSpace As NameSpace
    Set NameSpace = CentricApp.Outlook.Session
    
    AddMAPIDefaultFolder FolderList, NameSpace, olFolderConflicts
    AddMAPIDefaultFolder FolderList, NameSpace, olFolderDeletedItems
    AddMAPIDefaultFolder FolderList, NameSpace, olFolderDrafts
    AddMAPIDefaultFolder FolderList, NameSpace, olFolderJournal
    AddMAPIDefaultFolder FolderList, NameSpace, olFolderJunk
    AddMAPIDefaultFolder FolderList, NameSpace, olFolderLocalFailures
    AddMAPIDefaultFolder FolderList, NameSpace, olFolderNotes
    AddMAPIDefaultFolder FolderList, NameSpace, olFolderServerFailures
    AddMAPIDefaultFolder FolderList, NameSpace, olFolderSyncIssues
    AddMAPIDefaultFolder FolderList, NameSpace, olPublicFoldersAllPublicFolders
    
    GetDefaultIgnoreFolders = FolderList.AsString
End Function

Private Sub AddMAPIDefaultFolder(ByVal list As FolderList, ByVal NameSpace As NameSpace, ByVal ID As Outlook.OlDefaultFolders)
    On Error Resume Next
    Dim Folder As MAPIFolder
    Set Folder = NameSpace.GetDefaultFolder(ID)
    If Folder Is Nothing Then Exit Sub
    list.Add Folder.folderPath
End Sub

